import uuid

from typing import Optional

from pydantic import BaseModel, Field

from . import to_camel
from .cost_centre import CostCentreLink


class AccountLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: Optional[str]

    class Config:
        fields = {"id_": "id"}


class AccountBase(BaseModel):
    name: str = Field(..., min_length=1)
    is_starred: bool
    is_active: bool
    cost_centre: CostCentreLink

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class AccountIn(AccountBase):
    type_: int
    is_reconcilable: bool

    class Config:
        alias_generator = to_camel


class Account(AccountIn):
    id_: uuid.UUID
    code: int
    is_fixture: bool


class AccountBlank(AccountIn):
    name: str
    is_fixture: bool

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
