import uuid

from datetime import date, datetime
from typing import Union

from pydantic import BaseModel, Field, validator

from . import to_camel


class PeriodIn(BaseModel):
    valid_from: date
    valid_till: date

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y") if v is not None else None}

    @validator("valid_from", pre=True)
    def parse_valid_from(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("valid_till", pre=True)
    def parse_valid_till(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()


class Period(PeriodIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}


class PeriodLink(BaseModel):
    id_: uuid.UUID = Field(...)

    class Config:
        fields = {"id_": "id"}
