import uuid

from decimal import Decimal
from typing import List, Optional

from pydantic import BaseModel

from . import to_camel
from .period import PeriodLink
from .product import ProductLink
from .recipe_item import RecipeItem


class RecipeIn(BaseModel):
    sku: ProductLink

    recipe_yield: Decimal
    cost_price: Decimal
    sale_price: Decimal
    notes: str

    period: PeriodLink

    items: List[RecipeItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class Recipe(RecipeIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class RecipeBlank(RecipeIn):
    sku: Optional[ProductLink]  # type: ignore[assignment]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
