from datetime import date, datetime
from decimal import Decimal
from typing import List, Optional, Union

from pydantic import validator
from pydantic.main import BaseModel

from . import to_camel


class TrialBalanceItem(BaseModel):
    type_: Optional[str]
    name: Optional[str]
    debit: Optional[Decimal]
    credit: Optional[Decimal]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class TrialBalance(BaseModel):
    date_: date
    body: List[TrialBalanceItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
