import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { Purchases } from './purchases';

const url = '/api/purchases';
const serviceName = 'PurchasesService';

@Injectable({
  providedIn: 'root',
})
export class PurchasesService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(startDate: string | null, finishDate: string | null): Observable<Purchases> {
    const startDateWithSlash = startDate ? `/${startDate}` : '';
    const finishDateWithSlash = finishDate ? `/${finishDate}` : '';
    return this.http
      .get<Purchases>(`${url}${startDateWithSlash}${finishDateWithSlash}`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Purchases>;
  }
}
