import { Component, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { TrialBalance } from './trial-balance';
import { TrialBalanceDataSource } from './trial-balance-datasource';

@Component({
  selector: 'app-trial-balance',
  templateUrl: './trial-balance.component.html',
  styleUrls: ['./trial-balance.component.css'],
})
export class TrialBalanceComponent implements OnInit {
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: TrialBalance = new TrialBalance();
  dataSource: TrialBalanceDataSource = new TrialBalanceDataSource(this.info.body);
  form: FormGroup<{
    date: FormControl<Date>;
  }>;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['type', 'name', 'debit', 'credit'];

  constructor(private route: ActivatedRoute, private router: Router) {
    this.form = new FormGroup({
      date: new FormControl(new Date(), { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: TrialBalance };

      this.info = data.info;
      this.form.setValue({
        date: moment(this.info.date, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new TrialBalanceDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['trial-balance', info.date]);
  }

  getInfo(): TrialBalance {
    const formModel = this.form.value;

    return new TrialBalance({
      date: moment(formModel.date).format('DD-MMM-YYYY'),
    });
  }
}
